# internal imports
from bdot import carray_ext

import bdot

# external imports
import numpy as np
import bcolz

class carray(bcolz.carray):

	def dot(self, matrix, out=None):
		'''
			Dot product of two arrays, with bcolz.carray support. If out is provided it must
			match the output which would be constructed exactly, or an error will be
			raised.

			if you want to use the out parameter, but aren't sure how, use
			`bdot.carray.empty_dot()`

		Arguments:
			matrix (carray): two dimensional matrix in a bcolz.carray, row vector format or
			a one dimensional numpy.ndarray

			out: named parameter to be used as output
		'''

		# check dtype compatibility
		if self.dtype.type != matrix.dtype.type:
			raise ValueError("inputs must have the same dtype. Found {0} and {1}".format(self.dtype, matrix.dtype))

		# check shape
		if( (self.shape[1] != matrix.shape[0]) and 
			(type(matrix) == carray) and (self.shape[1] != matrix.shape[1])):
			raise ValueError("inputs must have compatible shapes. Found {0} and {1}".format(self.shape, matrix.shape))

		if type(matrix) == np.ndarray:

			assert len(matrix.shape) == 1

			# create output container, or check existing one
			if out == None:
				out =  np.empty(self.shape[0], dtype=self.dtype)
			else:
				assert len(out.shape) == 1

			if type(out) == np.ndarray:
				assert out.shape[0] == self.shape[0]
				carray_ext._dot(self, matrix, out)

				return out
			else:
				# output carray
				assert isinstance(out, bcolz.carray)

				carray_ext._dot_carray(self, matrix, out)
				return out
		else:

			assert len(matrix.shape) == 2

			# create output container, or check existing one
			if out == None:
				out = self.empty_like_dot(matrix)
			else:
				assert isinstance(out, bcolz.carray)
				assert len(out.shape) == 2
				assert out.shape[0] == self.shape[0]
				assert out.shape[1] == matrix.shape[0]

			# output carray
			carray_ext._dot_mat_carray(self, matrix, matrix[0], out)

			return out

	def empty_like_dot(self, matrix, chunklen=None, cparams=None, rootdir=None):
		'''
		Create en empty bdot.carray for use with the out parameter
		of the dot method. Allows saving to disk, selection of
		compression ratio and modification of other carray parameters.

		This is a relatively cheap operation.
		'''
		if type(matrix) == np.ndarray:

			assert len(matrix.shape) == 1

			return self.empty_like(shape=(self.shape[0],), chunklen=None, cparams=cparams, rootdir=rootdir)

		elif isinstance(matrix, bcolz.carray):

			assert len(matrix.shape) == 2

			return self.empty_like(shape=(self.shape[0], matrix.shape[0]), chunklen=None, cparams=cparams, rootdir=rootdir)



	def empty_like(self, shape=None, chunklen=None, cparams=None, rootdir=None):
		'''
		Create an empty bdot.carray container matching this one, with optional
		modifications.
		'''

		p_dtype = self.dtype
		if shape == None:
			shape = self.shape
		if cparams == None:
			cparams = self.cparams

		if(len(shape) == 1):

			result_template = np.ndarray(shape=(0), dtype=p_dtype)
			return bdot.carray(result_template, expectedlen=shape[0], chunklen=chunklen, cparams=cparams, rootdir=rootdir)


		elif(len(self.shape) == 2):

			result_template = np.ndarray((0, shape[1]), dtype=p_dtype)
			return bdot.carray(result_template, expectedlen=shape[0], chunklen=chunklen, cparams=cparams, rootdir=rootdir)


		else:
			raise ValueError("Can't create a carray like that. Only one and two dimensions supported.")




import nose
from nose.tools import raises

import bdot
import numpy as np
from numpy.testing import assert_array_equal, assert_array_almost_equal

# ndarray
def test_dot_int64():

	matrix = np.random.random_integers(0, 12000, size=(10000, 100))
	bcarray = bdot.carray(matrix, chunklen=2**13, cparams=bdot.cparams(clevel=2))

	v = bcarray[0]

	result = bcarray.dot(v)
	expected = matrix.dot(v)

	assert_array_equal(expected, result)


def test_dot_int32():

	matrix = np.random.random_integers(0, 12000, size=(10000, 100)).astype('int32')
	bcarray = bdot.carray(matrix, chunklen=2**13, cparams=bdot.cparams(clevel=2))

	v = bcarray[0]

	result = bcarray.dot(v)
	expected = matrix.dot(v)

	assert_array_equal(expected, result)


def test_dot_float64():

	matrix = np.random.random_sample(size=(10000, 100))
	bcarray = bdot.carray(matrix, chunklen=2**13, cparams=bdot.cparams(clevel=2))

	v = bcarray[0]

	result = bcarray.dot(v)
	expected = matrix.dot(v)

	assert_array_almost_equal(expected, result, decimal=5)


def test_dot_float32():

	matrix = np.random.random_sample(size=(10000, 100)).astype('float32')
	bcarray = bdot.carray(matrix, chunklen=2**13, cparams=bdot.cparams(clevel=2))

	v = bcarray[0]

	result = bcarray.dot(v)
	expected = matrix.dot(v)

	assert_array_almost_equal(expected, result, decimal=5)

# 1-D carray

def test_dot_matrix_1_int64():

	matrix = np.random.random_integers(0, 120, size=(10000, 100))
	bcarray = bdot.carray(matrix, chunklen=2**13, cparams=bdot.cparams(clevel=2))

	v = bcarray[0]

	output = bcarray.empty_like_dot(v)

	result = bcarray.dot(v, out=output)
	expected = matrix.dot(v)

	assert_array_equal(expected, result)

# carray
def test_dot_matrix_int64():

	matrix = np.random.random_integers(0, 120, size=(1000, 100))
	bcarray1 = bdot.carray(matrix, chunklen=2**9, cparams=bdot.cparams(clevel=2))
	bcarray2 = bdot.carray(matrix, chunklen=2**9, cparams=bdot.cparams(clevel=2))


	result = bcarray1.dot(bcarray2)
	expected = matrix.dot(matrix.T)

	assert_array_equal(expected, result)


def test_dot_matrix_int64_unequal_chunklen():

	matrix1 = np.random.random_integers(0, 120, size=(1000, 100))
	bcarray1 = bdot.carray(matrix1, chunklen=2**9, cparams=bdot.cparams(clevel=2))
	matrix2 = np.random.random_integers(0, 120, size=(1000, 100))
	bcarray2 = bdot.carray(matrix2, chunklen=2**8, cparams=bdot.cparams(clevel=2))


	result = bcarray1.dot(bcarray2)
	expected = matrix1.dot(matrix2.T)

	assert_array_equal(expected, result)


def test_dot_matrix_int64_unequal_length():

	matrix1 = np.random.random_integers(0, 120, size=(1000, 100))
	bcarray1 = bdot.carray(matrix1, chunklen=2**9, cparams=bdot.cparams(clevel=2))
	matrix2 = np.random.random_integers(0, 120, size=(10000, 100))
	bcarray2 = bdot.carray(matrix2, chunklen=2**10, cparams=bdot.cparams(clevel=2))


	result = bcarray1.dot(bcarray2)
	expected = matrix1.dot(matrix2.T)

	assert_array_equal(expected, result)

def test_dot_matrix_int32():

	matrix = np.random.random_integers(0, 120, size=(1000, 100)).astype('int32')
	bcarray1 = bdot.carray(matrix, chunklen=2**9, cparams=bdot.cparams(clevel=2))
	bcarray2 = bdot.carray(matrix, chunklen=2**9, cparams=bdot.cparams(clevel=2))


	result = bcarray1.dot(bcarray2)
	expected = matrix.dot(matrix.T)

	assert_array_equal(expected, result)


def test_dot_matrix_float64():

	matrix = np.random.random_sample(size=(1000, 100))
	bcarray1 = bdot.carray(matrix, chunklen=2**9, cparams=bdot.cparams(clevel=2))
	bcarray2 = bdot.carray(matrix, chunklen=2**9, cparams=bdot.cparams(clevel=2))


	result = bcarray1.dot(bcarray2)
	expected = matrix.dot(matrix.T)

	assert_array_almost_equal(expected, result, decimal=5)


def test_dot_matrix_float32():

	matrix = np.random.random_sample(size=(1000, 100)).astype('float32')
	bcarray1 = bdot.carray(matrix, chunklen=2**9, cparams=bdot.cparams(clevel=2))
	bcarray2 = bdot.carray(matrix, chunklen=2**9, cparams=bdot.cparams(clevel=2))


	result = bcarray1.dot(bcarray2)
	expected = matrix.dot(matrix.T)

	assert_array_almost_equal(expected, result, decimal=4)


@raises(ValueError)
def test_dot_incompatible_dtype():

	matrix = np.random.random_integers(0, 12000, size=(10000, 100))
	bcarray = bdot.carray(matrix, chunklen=2**13, cparams=bdot.cparams(clevel=2))

	v = bcarray[0].astype('int32')

	result = bcarray.dot(v)

'''
@raises(ValueError)
def test_dot_incompatible_shapes():

	matrix = np.random.random_integers(0, 12000, size=(10000, 101))
	bcarray = bdot.carray(matrix[:, :100], chunklen=2**13, cparams=bdot.cparams(clevel=2))


	result = bcarray.dot(matrix)
'''



import numpy as np
cimport numpy as np

import cython
cimport cython
import bdot
from bcolz.carray_ext cimport carray, chunk


# numpy optimizations from:
# http://docs.cython.org/src/tutorial/numpy.html

# fused types (templating) from
# http://docs.cython.org/src/userguide/fusedtypes.html

ctypedef fused numpy_native_number:
	np.int64_t
	np.int32_t
	np.float64_t
	np.float32_t


@cython.wraparound(False)
@cython.boundscheck(False)
cpdef _dot(carray matrix, np.ndarray[numpy_native_number, ndim=1] vector, np.ndarray[numpy_native_number, ndim=1] result):
	'''
		Calculate dot product between a bcolz.carray matrix and a numpy vector.
		Second dimension of matrix must match first dimension of vector.
		
		Arguments:
			matrix (carray): two dimensional matrix in a bcolz.carray, row vector format
			vector (ndarray): one dimensional vector in a numpy array
		
		Returns:
			ndarray: result of dot product, one value per row in orginal matrix
	'''

	# fused type conversion
	if numpy_native_number is np.int64_t:
		p_dtype = np.int64
	elif numpy_native_number is np.int32_t:
		p_dtype = np.int32
	elif numpy_native_number is np.float64_t:
		p_dtype = np.float64
	else:
		p_dtype = np.float32

	cdef np.ndarray[numpy_native_number] dot_i = np.empty(matrix.chunklen, dtype=p_dtype)

	cdef np.ndarray[numpy_native_number, ndim=2] m_i = np.empty((matrix.chunklen, matrix.shape[1]), dtype=p_dtype)


	cdef chunk chunk_

	cdef Py_ssize_t i, chunk_start, chunk_len, leftover_len
	cdef unsigned int result_j, j

	chunk_len = matrix.chunklen

	leftover_len = cython.cdiv(matrix.leftover, matrix.atomsize)


	for i in range(matrix.nchunks):
		chunk_ = matrix.chunks[i]

		chunk_._getitem(0, chunk_len, m_i.data)
		dot_i = np.dot(m_i, vector)

		# copy to result
		chunk_start = i * chunk_len
		for j in range(chunk_len):
			result_j = <unsigned int> (chunk_start + j)
			result[result_j] = dot_i[j]

	if leftover_len > 0:
		dot_i = np.dot(matrix.leftover_array, vector)

		chunk_start = (i + 1) * chunk_len
		for j in range(leftover_len):
			result_j = <unsigned int> (chunk_start + j)
			result[result_j] = dot_i[j]


@cython.wraparound(False)
@cython.boundscheck(False)
cpdef _dot_carray(carray matrix, np.ndarray[numpy_native_number, ndim=1] vector, carray c_result):
	'''
		Calculate dot product between a bcolz.carray matrix and a numpy vector, storing the results
		in a carray.
		Second dimension of matrix must match first dimension of vector.
		
		Arguments:
			matrix (carray): two dimensional matrix in a bcolz.carray, row vector format
			vector (ndarray): one dimensional vector in a numpy array
		
		Returns:
			ndarray: result of dot product, one value per row in orginal matrix
	'''

	# fused type conversion
	if numpy_native_number is np.int64_t:
		p_dtype = np.int64
	elif numpy_native_number is np.int32_t:
		p_dtype = np.int32
	elif numpy_native_number is np.float64_t:
		p_dtype = np.float64
	else:
		p_dtype = np.float32

	cdef Py_ssize_t i, chunk_start, chunk_len, leftover_len

	chunk_len = matrix.chunklen

	cdef np.ndarray[numpy_native_number] dot_i = np.empty(matrix.chunklen, dtype=p_dtype)

	cdef np.ndarray[numpy_native_number, ndim=2] m_i = np.empty((matrix.chunklen, matrix.shape[1]), dtype=p_dtype)


	cdef chunk chunk_


	leftover_len = cython.cdiv(matrix.leftover, matrix.atomsize)


	for i in range(matrix.nchunks):
		chunk_ = matrix.chunks[i]

		chunk_._getitem(0, chunk_len, m_i.data)
		dot_i = np.dot(m_i, vector)

		#write new chunk to result carray
		c_result.append(dot_i)

	if leftover_len > 0:
		dot_i = np.dot(matrix.leftover_array, vector)

		#write new chunk to result carray
		c_result.append(dot_i[:leftover_len])


@cython.wraparound(False)
@cython.boundscheck(False)
cpdef _dot_mat(carray m1, carray m2, np.ndarray[numpy_native_number, ndim=1] type_indicator):
	'''
		Calculate matrix multiply between bcolz.carray matrix and transpose of
		a second bcolz.carray matrix.
		Second dimension of m1 must match second dimension of m2.
		
		Requires that resulting matrix fit in RAM.

		Requires that chunks and matrix multiply of chunks fit in RAM.

		Arguments:
			m1 (carray): two dimensional matrix in a bcolz.carray, row vector format
			m2 (carray): two dimensional matrix in a bcolz.carray, row vector format
			type_indicator(ndarray) : hack to allow use of fused types (just pass in the first row)
		
		Returns:
			carray: result of matrix multiply between m1 and m2.T, matrix with dimensions equal to 
			first dimension of m1 by first dimension of m2

	'''

	# fused type conversion
	if numpy_native_number is np.int64_t:
		p_dtype = np.int64
	elif numpy_native_number is np.int32_t:
		p_dtype = np.int32
	elif numpy_native_number is np.float64_t:
		p_dtype = np.float64
	else:
		p_dtype = np.float32

	cdef Py_ssize_t i, chunk_start_i, chunk_len_i, leftover_len_i
	cdef Py_ssize_t j, chunk_start_j, chunk_len_j, leftover_len_j

	# iterate through m1 in outer loop, to facilitate later chunking into output carray
	chunk_len_i = m1.chunklen
	chunk_len_j = m2.chunklen


	cdef np.ndarray[numpy_native_number, ndim=2] m_i = np.empty((chunk_len_i, m1.shape[1]), dtype=p_dtype)

	cdef np.ndarray[numpy_native_number, ndim=2] m_j = np.empty((chunk_len_j, m2.shape[1]), dtype=p_dtype)

	cdef np.ndarray[numpy_native_number, ndim=2] dot_k = np.empty((chunk_len_i, chunk_len_j), dtype=p_dtype)

	cdef np.ndarray[numpy_native_number, ndim=2] result = np.empty((m1.shape[0], m2.shape[0]), dtype=p_dtype)

	cdef chunk chunk_i_
	cdef chunk chunk_j_

	cdef unsigned int result_k, k
	cdef unsigned int result_l, l


	leftover_len_i = cython.cdiv(m1.leftover, m1.atomsize)
	leftover_len_j = cython.cdiv(m2.leftover, m2.atomsize)


	for i in range(m1.nchunks):

		chunk_i_ = m1.chunks[i]
		chunk_i_._getitem(0, chunk_len_i, m_i.data)


		for j in range(m2.nchunks):
			chunk_j_ = m2.chunks[j]

			chunk_j_._getitem(0, chunk_len_j, m_j.data)

			dot_k = np.dot(m_i, m_j.T)

			# copy to result
			chunk_start_i = i * chunk_len_i
			chunk_start_j = j * chunk_len_j
			for k in range(chunk_len_i):
				result_k = <unsigned int> (chunk_start_i + k)
				for l in range(chunk_len_j):
					result_l = <unsigned int> (chunk_start_j + l)
					result[result_k, result_l] = dot_k[k, l]

		# do last chunk in first array
		if leftover_len_j > 0:
			dot_k = np.dot(m_i, m2.leftover_array.T)

			chunk_start_i = i * chunk_len_i
			chunk_start_j = (j + 1) * chunk_len_j
			for k in range(chunk_len_i):
				result_k = <unsigned int> (chunk_start_i + k)
				for l in range(leftover_len_j):
					result_l = <unsigned int> (chunk_start_j + l)
					result[result_k, result_l] = dot_k[k, l]


	# do last chunk in second array
	if leftover_len_i > 0:

		for j in range(m2.nchunks):

			chunk_j_ = m2.chunks[j]

			chunk_j_._getitem(0, chunk_len_j, m_j.data)

			dot_k = np.dot(m1.leftover_array, m_j.T)

			# copy to result
			chunk_start_i = (i + 1) * chunk_len_i
			chunk_start_j = j * chunk_len_j
			for k in range(leftover_len_i):
				result_k = <unsigned int> (chunk_start_i + k)
				for l in range(chunk_len_j):
					result_l = <unsigned int> (chunk_start_j + l)
					result[result_k, result_l] = dot_k[k, l]


		# do last chunk in first array
		if leftover_len_j > 0:
			dot_k = np.dot(m1.leftover_array, m2.leftover_array.T)

			chunk_start_i = (i + 1) * chunk_len_i
			chunk_start_j = (j + 1) * chunk_len_j
			for k in range(leftover_len_i):
				result_k = <unsigned int> (chunk_start_i + k)
				for l in range(leftover_len_j):
					result_l = <unsigned int> (chunk_start_j + l)
					result[result_k, result_l] = dot_k[k, l]

	return result


@cython.wraparound(False)
@cython.boundscheck(False)
cpdef _dot_mat_carray(carray m1, carray m2, np.ndarray[numpy_native_number, ndim=1] type_indicator, carray c_result):
	'''
		Calculate matrix multiply between bcolz.carray matrix and transpose of
		a second bcolz.carray matrix.
		Second dimension of m1 must match second dimension of m2.

		Requires that chunks and matrix multiply of chunks fit in RAM.

		Arguments:
			m1 (carray): two dimensional matrix in a bcolz.carray, row vector format
			m2 (carray): two dimensional matrix in a bcolz.carray, row vector format
			type_indicator(ndarray) : hack to allow use of fused types (just pass in the first row)
		
		Returns:
			carray: result of matrix multiply between m1 and m2.T, matrix with dimensions equal to 
			first dimension of m1 by first dimension of m2

	'''

	# fused type conversion
	if numpy_native_number is np.int64_t:
		p_dtype = np.int64
	elif numpy_native_number is np.int32_t:
		p_dtype = np.int32
	elif numpy_native_number is np.float64_t:
		p_dtype = np.float64
	else:
		p_dtype = np.float32

	cdef Py_ssize_t i, chunk_start_i, chunk_len_i, leftover_len_i
	cdef Py_ssize_t j, chunk_start_j, chunk_len_j, leftover_len_j

	# iterate through m1 in outer loop, to facilitate later chunking into output carray
	chunk_len_i = m1.chunklen
	chunk_len_j = m2.chunklen


	cdef np.ndarray[numpy_native_number, ndim=2] m_i = np.empty((chunk_len_i, m1.shape[1]), dtype=p_dtype)

	cdef np.ndarray[numpy_native_number, ndim=2] m_j = np.empty((chunk_len_j, m2.shape[1]), dtype=p_dtype)

	cdef np.ndarray[numpy_native_number, ndim=2] dot_k = np.empty((chunk_len_i, chunk_len_j), dtype=p_dtype)

	cdef np.ndarray[numpy_native_number, ndim=2] result_i = np.empty((chunk_len_i, m2.shape[0]), dtype=p_dtype)


	cdef chunk chunk_i_
	cdef chunk chunk_j_

	cdef unsigned int result_k, k
	cdef unsigned int result_l, l


	leftover_len_i = cython.cdiv(m1.leftover, m1.atomsize)
	leftover_len_j = cython.cdiv(m2.leftover, m2.atomsize)


	for i in range(m1.nchunks):

		chunk_i_ = m1.chunks[i]
		chunk_i_._getitem(0, chunk_len_i, m_i.data)

		chunk_start_i = i * chunk_len_i

		for j in range(m2.nchunks):
			chunk_j_ = m2.chunks[j]

			chunk_j_._getitem(0, chunk_len_j, m_j.data)

			dot_k = np.dot(m_i, m_j.T)

			# copy to result
			chunk_start_j = j * chunk_len_j
			for k in range(chunk_len_i):
				for l in range(chunk_len_j):
					result_l = <unsigned int> (chunk_start_j + l)
					result_i[k, result_l] = dot_k[k, l]

		# do last chunk in second array
		if leftover_len_j > 0:
			dot_k = np.dot(m_i, m2.leftover_array.T)

			chunk_start_j = (j + 1) * chunk_len_j
			for k in range(chunk_len_i):
				for l in range(leftover_len_j):
					result_l = <unsigned int> (chunk_start_j + l)
					result_i[k, result_l] = dot_k[k, l]

		#write new chunk to result carray
		c_result.append(result_i) # fill_chunks(self, object array_)


	# do last chunk in first array
	if leftover_len_i > 0:

		for j in range(m2.nchunks):

			chunk_j_ = m2.chunks[j]

			chunk_j_._getitem(0, chunk_len_j, m_j.data)

			dot_k = np.dot(m1.leftover_array, m_j.T)

			# copy to result
			chunk_start_i = (i + 1) * chunk_len_i
			chunk_start_j = j * chunk_len_j
			for k in range(leftover_len_i):
				for l in range(chunk_len_j):
					result_l = <unsigned int> (chunk_start_j + l)
					result_i[k, result_l] = dot_k[k, l]


		# do last chunk in both arrays
		if leftover_len_j > 0:
			dot_k = np.dot(m1.leftover_array, m2.leftover_array.T)

			chunk_start_i = (i + 1) * chunk_len_i
			chunk_start_j = (j + 1) * chunk_len_j
			for k in range(leftover_len_i):
				for l in range(leftover_len_j):
					result_l = <unsigned int> (chunk_start_j + l)
					result_i[k, result_l] = dot_k[k, l]

		#write new chunk to result carray
		c_result.append(result_i[:leftover_len_i]) # fill_chunks(self, object array_)





from distutils.core import Extension
from distutils.core import setup




# The minimum version of Cython required for generating extensions
min_cython_version = '0.20'

try:
    import Cython
    cur_cython_version = Cython.__version__
    from Cython.Distutils import build_ext
except:
    exit_with_error(
        "You need %(pkgname)s %(pkgver)s or greater to compile bcolz!"
        % {'pkgname': 'Cython', 'pkgver': min_cython_version})

if cur_cython_version < min_cython_version:
    exit_with_error(
        "At least Cython %s is needed so as to generate extensions!"
        % (min_cython_version))
else:
    print("* Found %(pkgname)s %(pkgver)s package installed."
          % {'pkgname': 'Cython', 'pkgver': cur_cython_version})

########### Project specific command line options ###########

class bdot_build_ext(build_ext):
    user_options = build_ext.user_options

    def initialize_options(self):
        self.from_templates = False
        build_ext.initialize_options(self)

    def run(self):

        build_ext.run(self)


######### End project specific command line options #########


VERSION = open('VERSION').read().strip()
# Create the version.py file
open('bdot/version.py', 'w').write('__version__ = "%s"\n' % VERSION)



# Sources & libraries
inc_dirs = ['bdot']
lib_dirs = []
libs = []
def_macros = []
sources_carray = ["bdot/carray_ext.pyx"]

# Include NumPy header dirs
from numpy.distutils.misc_util import get_numpy_include_dirs

inc_dirs.extend(get_numpy_include_dirs())

long_desc = """
Bdot does big dot products (by making your RAM bigger on the inside). 
It aims to let you do all the things you want to do with numpy, 
but don't have enough memory for. You can also use it to save space
in the cloud and quickly build production microservices.

It's pretty math.

Bdot is based on Bcolz and includes transparent disk-based storage. It was created 
to simplify production implementation of nearest neighbor search and other fancy 
machine learning rigmarole.
"""

setup(
    name = 'bdot',
    version = VERSION,
    description = 'Fast Matrix Multiply on Pretty Big Data (powered by Bcolz)',
    long_description= long_desc,
    url='https://github.com/tailwind/bdot',
    author='Waylon Flinn',
    cmdclass={'build_ext' : bdot_build_ext},
    ext_modules = [
          Extension("bdot.carray_ext",
                    include_dirs=inc_dirs,
                    sources=sources_carray)
          ],
    packages=['bdot', 'bdot.tests'],
    data_files=[('.' , ['VERSION'])],
    install_requires=['cython>=0.20', 'numpy>=1.7.0', 'bcolz>=0.9.0'],
    license='MIT',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Intended Audience :: Science/Research',
        'Topic :: Scientific/Engineering',
        'Topic :: Software Development',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4'
    ],
    keywords='dot product matrix multiply compression bcolz numpy'
)
